from pipedrive.activities import Activities as BaseActivities


class Activities(BaseActivities):
    def __init__(self, client):
        super().__init__(client)

    def get_activity_types(self, **kwargs):
        url = "activityTypes"
        return self._client._get(self._client.BASE_URL + url, **kwargs)



class Files:
    def __init__(self, client):
        self._client = client

    def get_all_files(self, params=None, **kwargs):
        url = "files"
        return self._client._get(self._client.BASE_URL + url, params=params, **kwargs)

    def update_file(self, file_id, data, **kwargs):
        url = "files/{}".format(file_id)
        return self._client._put(self._client.BASE_URL + url, json=data, **kwargs)



import win32com.client as win32
import datetime
import re

def get_folder_by_name(email, folder_name=""):
    """
    Search for a calendar folder in some email account, logged in outlook app
    NOTE: the folder must be inside the calendar folder, leave 'folder_name' empty to use de default calendar folder

    Example
    -------
    get_folder_by_name('example@example.com', 'Fees and coupons')

    return: COM outlook folder object
    """

    outlook = win32.Dispatch('outlook.application')
    recip = outlook.Session.CreateRecipient(email)

    if folder_name != "":
        for folder in outlook.Session.GetSharedDefaultFolder(recip, 9).Folders:
            if (folder.name == folder_name):
                return folder
    else:
        return outlook.Session.GetSharedDefaultFolder(recip, 9)

    raise Exception("Email or folder not found")


def search_metting_by_title_on_date(subject, email, date, folder_name=""):
    """
    Search for a metting by date and subject title
    NOTE: the folder must be inside the calendar folder, leave 'folder_name' empty to use de default calendar folder

    Example
    -------
    teste.search_metting_by_title_on_date(
        'Event title',
        'example@example.com',
        datetime.date(2020,12,5),
        'My tests folder'
    )

    Return the metting if found or false
    """

    calendar_folder = get_folder_by_name(email, folder_name)

    for item in calendar_folder.Items:
        metting_start = datetime.date(item.start.year, item.start.month, item.start.day)

        if metting_start == date and item.subject == subject:
            return item

    return False


def create_allday_metting(subject, date, body, recipients, sender, folder_name="", send=False, reminder_minutes=10080):
    """
    Create a meeting with all day enabled in a given calendar folder and account
    This function does not repeat mettings, based on 'date' and 'subject'.
    This mean if you create twice, the second one will be send to recipients if some recipient or body change
    NOTE: the folder must be inside the calendar folder, leave 'folder_name' empty to use de default calendar folder

    Example
    -------
    create_allday_metting(
        "Event title",
        datetime.date(2020, 10, 25),
        "My body\nwith line break",
        ["email@hotmail.com", "email2@hotmail.com"], # Just work for outlook or exchange e-mails
        "example@example.com",
        "My tests folder",
        True #True to Send, false to display
    )
    """

    created_metting = search_metting_by_title_on_date(subject, sender, date, folder_name)

    # Create the metting
    if not created_metting:
        calendar_folder = get_folder_by_name(sender, folder_name)
        appointment = calendar_folder.Items.add

        appointment.Subject = subject
        appointment.Start = date.strftime('%Y-%m-%d') + " 08:00"
        appointment.End = date.strftime('%Y-%m-%d') + " 08:30"
        appointment.AllDayEvent = True
        appointment.MeetingStatus = 1
    else:
        appointment = created_metting

    new_recipients = [sender] + recipients
    old_recipients = []
    to_remove = []
    to_add = []

   # Set recipients to be removed
    for idx,r in enumerate(appointment.Recipients):

        # Just work for outlook or exchange e-mails
        addr = r.PropertyAccessor.GetProperty('http://schemas.microsoft.com/mapi/proptag/0x39FE001E')

        # Outlook API remove from index starting by 1
        if addr not in new_recipients:
            to_remove.append(idx+1)

        old_recipients.append(addr)

    to_add = list(set(new_recipients) - set(old_recipients))

    # If nothing change (recipients or body) not send, just return
    if (not to_add) and (not to_remove) and (re.sub(r'\s', '', appointment.Body) == re.sub(r'\s', '', body)):
        return

    # Add or update the body
    appointment.Body = body

    # Remove the recipients
    for idx in to_remove:
        appointment.Recipients.Remove(idx)

    # Add the recipients to share the event
    for email in to_add:
        appointment.Recipients.Add(email)

    # Set the reminder
    appointment.ReminderOverrideDefault = True
    appointment.ReminderSet = True
    appointment.ReminderMinutesBeforeStart = reminder_minutes

    if send:
        appointment.Send()
    else:
        appointment.Display()



class RedeCustomer():
    """
    Access 'rede customers' api.
    Have some methods to direct access some endpoints.
    """

    def __init__(self, client):
        self.client = client
        self.base_url = 'api.xpi.com.br/rede-customer'
        self.subscription_key = 'b63f9758680845afb25dbd47a5e95c4a'


    def get_from_url(self, api_path, params={}):
        '''
        Get something from the url 'api.xpi.com.br/rede-customer'.
        You don't need to pass the base route, only the path for this api.
        The path must start with '/'.

        e.g.: get_from_url('/v2/customers/{customer_code}/positions', params)
        '''

        full_url = self.base_url + api_path
        return self.client.request(full_url, params, self.subscription_key)


    def get_customer_positions(self, xp_code, params={}):
        '''
        Get customer positions from xp api

        e.g.: get_customer_positions(
            123321,                     # Customer xp code. Can be string or int
            {
                'createCache': True      # We thing that this force xp api to cache this request
            }
        )
        '''
        return self.get_from_url(
            '/v2/customers/{}/positions'.format(str(xp_code)),
            params
        )



from pipedrive.deals import Deals as BaseDeals


class Deals(BaseDeals):
    def __init__(self, client):
        super().__init__(client)

    def delete_participant_to_deal_by_person_id(self, deal_id, person_id):
        find_participant_funct = lambda x: str(x["person"]["id"]) == str(person_id)

        all_participants = self.get_deal_participants(str(deal_id))["data"]
        filtered_participant = list(filter(find_participant_funct, all_participants))

        if len(filtered_participant) == 1:
            return self.delete_participant_to_deal(
                deal_id, filtered_participant[0]["id"]
            )
        else:
            raise Exception(
                "Not find participant id={} in the deal id={}".format(
                    person_id, deal_id
                )
            )

    def get_deal_one_field(self, field_id, **kwargs):
        url = "dealFields/{}".format(field_id)
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def update_deal_field(self, field_id, data, **kwargs):
        url = "dealFields/{}".format(field_id)
        return self._client._put(self._client.BASE_URL + url, json=data, **kwargs)



class RedeAdvisors():
    """
    Access 'rede advisor' api.
    Have some methods to direct access some endpoints.
    """

    def __init__(self, client):
        self.client = client
        self.base_url = 'api.xpi.com.br/rede-advisors'
        self.subscription_key = 'b63f9758680845afb25dbd47a5e95c4a'


    def get_from_url(self, api_path, params={}):
        '''
        Get something from the url 'api.xpi.com.br/rede-advisors'.
        You don't need to pass the base route, only the path for this api.
        The path must start with '/'.

        e.g.: get_from_url('/v1/advisors/customer-wallets', params)
        '''

        full_url = self.base_url + api_path
        return self.client.request(full_url, params, self.subscription_key)

    def get_customer_wallet(self, params={}):
        '''
        Get customer wallet from xp api

        e.g.: get_customer_wallet({
            'limit': 0              # the param limit = 0 return all the customers (if you get an 404, try change the limit to 100)
        })
        '''
        return self.get_from_url('/v1/advisors/customer-wallets', params)

    def get_current_portfolios(self, params={}):
        return self.get_from_url('/v1/advisors/current-portfolios', params)



class InvestmentFunds():
    """
    Access 'investment funds' api.
    Have some methods to direct access some endpoints.
    """

    def __init__(self, client):
        self.client = client
        self.base_url = 'api.xpi.com.br/investment-funds'
        self.subscription_key = '602224c06c434f35b6352ca902c5020a'


    def get_from_url(self, api_path, params={}):
        '''
        Get something from the url 'api.xpi.com.br/investment-funds'.
        You don't need to pass the base route, only the path for this api.
        The path must start with '/'.

        e.g.: get_from_url(
            '/yield-rede/v2/investment-funds-rede',
            params
        )
        '''

        full_url = self.base_url + api_path
        return self.client.request(full_url, params, self.subscription_key)


    def get_hub_funds(self, params={}):
        '''
        Get all funds from xp hub page
        website ref.:

        e.g.: get_hub_funds(params)
        '''
        return self.get_from_url('/yield-rede/v2/investment-funds-rede')


    def get_open_funds(self, params={}):
        '''
        Get all funds from xp open page
        website ref.: https://www.xpi.com.br/investimentos/fundos-de-investimento/lista/#/

        e.g.: get_open_funds({
            'onlyrpps': False,                  # We don't know for what is this params
            'family': None                      # but to work must be used
        })
        '''
        return self.get_from_url('/yield-portal/v2/investment-funds', params)



class PensionFunds():
    """
    Access 'pension funds' api.
    Have some methods to direct access some endpoints.
    """

    def __init__(self, client):
        self.client = client
        self.base_url = 'api.xpi.com.br/pension-funds'
        self.subscription_key = '6d05a3fbc85c49bc9368981fc193d468'


    def get_from_url(self, api_path, params={}):
        '''
        Get something from the url 'api.xpi.com.br/pension-funds'.
        You don't need to pass the base route, only the path for this api.
        The path must start with '/'.

        e.g.: get_from_url('/investment-funds', {
            'status': 1,                # we don't know for what is this params
            'brand': 3                  #
        })
        '''

        full_url = self.base_url + api_path
        return self.client.request(full_url, params, self.subscription_key)


    def get_prev(self, params={}):
        '''
        Get all prev funds from xp open page:
        website ref.: https://institucional.xpi.com.br/previdencia-privada/lista-de-fundos/#/

        e.g.: get_prev({
            'status': 1,                # we don't know for what is this params
            'brand': 3                  #
        })
        '''
        return self.get_from_url('/investment-funds', params)



import pickle
from loguru import logger


class Backup:
    def __init__(self, client):
        self._client = client

    def save_persons(self, path):
        data = self._client._get_all_from(self._client.persons.get_all_persons)

        pickle.dump(data, open(path, "wb"))
        logger.info("Saved in backup_persons.p {} persons".format(len(data)))

    def save_deals(self, path):
        data = self._client._get_all_from(self._client.deals.get_all_deals)

        pickle.dump(data, open(path, "wb"))
        logger.info("Saved in backup_deals.p {} deals".format(len(data)))

    def save_activities(self, path):
        data = self._client._get_all_from(
            self._client.activities.get_all_activities, {"user_id": 0}
        )

        pickle.dump(data, open(path, "wb"))
        logger.info("Saved in backup_activities.p {} activities".format(len(data)))

    def save_filters(self, path):
        data = self._client.filters.get_all_filters()["data"]

        pickle.dump(data, open(path, "wb"))
        logger.info("Saved in backup_filters.p {} filters".format(len(data)))

    def save_notes(self, path):
        data = self._client._get_all_from(self._client.notes.get_all_notes)

        pickle.dump(data, open(path, "wb"))
        logger.info("Saved in backup_notes.p {} notes".format(len(data)))

    def save_organizations(self, path):
        data = self._client._get_all_from(
            self._client.organizations.get_all_organizations
        )

        pickle.dump(data, open(path, "wb"))
        logger.info(
            "Saved in backup_organizations.p {} organizations".format(len(data))
        )

    def save_pipelines(self, path):
        data = self._client.pipelines.get_all_pipelines()['data']

        pickle.dump(data, open(path, "wb"))
        logger.info("Saved in backup_pipelines.p {} pipelines".format(len(data)))

    def save_products(self, path):
        data = self._client._get_all_from(self._client.products.get_all_products)

        pickle.dump(data, open(path, "wb"))
        logger.info("Saved in backup_products.p {} products".format(len(data)))

    def save_users(self, path):
        data = self._client.users.get_all_users()['data']

        pickle.dump(data, open(path, "wb"))
        logger.info("Saved in backup_users.p {} users".format(len(data)))

    def save_files(self, path):
        data = self._client.files.get_all_files()['data']

        pickle.dump(data, open(path, "wb"))
        logger.info("Saved in backup_files.p {} files".format(len(data)))



import pickle
from loguru import logger


class Manipulator:
    def __init__(self, client):
        self._client = client

    """
    Move the persons data to another persons, based on persons id, and keep they all as active as clone (Don't do auto merge)

    Parameters
    ----------
    ids_from: str or list(str)
        pipedrive person id to get person data

    ids_to: str or list(str)
        pipedrive person id to set person data

    exclude_fields: list(str)
        fields to not update

    Example
    --------
    # Move the data from person with id=1 to person with id=2, NOTE: at the end we will have 2 persons with the same data
    .move_person_data('1', '2')

    # Exchange person data with each other
    .move_person_data(['1','2'], ['2','1'])
    """

    def move_persons_data(
        self,
        ids_from,
        ids_to,
        exclude_fields=[
            "add_time",
            "visible_to",
            "org_name",
            "org_id",
            "label",
            "picture_id",
        ],
        must_use_pickle=False,
    ):
        id_persons_from, id_persons_to = self._sanitize_ids_from_to(ids_from, ids_to)

        exclude_field_list = exclude_fields + ["id"]
        unique_ids = self._get_unique_ids_from_to(ids_from, ids_to)

        if must_use_pickle:
            persons_data = pickle.load(open("persons_data.p", "rb"))
        else:
            persons_data = self._get_persons_by_ids(unique_ids, exclude_field_list)
            pickle.dump(persons_data, open("persons_data.p", "wb"))

        for i in range(0, len(id_persons_from)):
            person_from = id_persons_from[i]
            person_to = id_persons_to[i]
            person_data = persons_data[person_from]

            person_data["owner_id"] = person_data["owner_id"]["id"]

            res = self._client.persons.update_person(person_to, person_data)
            if res["success"] != True:
                raise Exception(
                    "Move data from id={} to id={} not succeed!\nResponse:\n{}".format(
                        person_from, person_to, res
                    )
                )

            logger.info(
                "Moved person id={} data to person id={}".format(person_from, person_to)
            )
            key = input("press enter to keep going: ")
            if key != "":
                return

        for id_person in unique_ids:
            res = self._client.persons.update_person(id_person, {"active_flag": True})
            if res["success"] != True:
                raise Exception(
                    "Set person id={} as active not succeed!\nResponse:\n{}".format(
                        id_person, res
                    )
                )

    """
    Move all person deal that match the date, to another person. can be used in batch, passing multiple persons
    NOTE: Pipe deals has the figure of participants (persons present in the deal, but no the central one).
          At this point of time the pipedrive DON'T auto change participants when changed the central person,
          so this function make the change and leaves the others participants unchanged

    Parameters
    ----------
    ids_from: str or list(str)
        pipedrive person id to get person data

    ids_to: str or list(str)
        pipedrive person id to set person data

    start_date: str
        filter deals for created from this date. Leave empty to min date

    end_date: str
        filter deals for created before this date. Leave empty to max date
    Example
    --------
    # Move the deals from person id=1 to person id=2
    .move_persons_deals('1', '2')

    # Move the deals created before 2019-08-08 from person id=1 to person id=2
    .move_persons_deals('1', '2', end_date='2019-08-08')

    # Exchange person deals with each other
    .move_persons_deals(['1','2'], ['2','1'])
    """

    def move_persons_deals(
        self,
        ids_from,
        ids_to,
        start_date="0000-00-00",
        end_date="9999-99-99",
        must_use_pickle=False,
    ):
        self._move_persons_records_to(
            "deals", ids_from, ids_to, start_date, end_date, must_use_pickle
        )

    """
    Move all person activities that match the date, to another person. can be used in batch, passing multiple persons.
    NOTE: Pipe activities has the figure of participants (persons present in the activity, but no the central one).
          At this point of time the pipedrive auto change participants when changed the central person, leaving the others participants unchanged

    Parameters
    ----------
    ids_from: str or list(str)
        pipedrive person id to get person data

    ids_to: str or list(str)
        pipedrive person id to set person data

    start_date: str
        filter deals for created from this date. Leave empty to min date

    end_date: str
        filter deals for created before this date. Leave empty to max date
    Example
    --------
    # Move the activities from person id=1 to person id=2
    .move_persons_activities('1', '2')

    # Move the activities created before 2019-08-08 from person id=1 to person id=2
    .move_persons_activities('1', '2', end_date='2019-08-08')

    # Exchange person deals with each other
    .move_persons_activities(['1','2'], ['2','1'])
    """

    def move_persons_activities(
        self,
        ids_from,
        ids_to,
        start_date="0000-00-00",
        end_date="9999-99-99",
        must_use_pickle=False,
    ):
        self._move_persons_records_to(
            "activities", ids_from, ids_to, start_date, end_date, must_use_pickle
        )

    def move_persons_notes(
        self,
        ids_from,
        ids_to,
        start_date="0000-00-00",
        end_date="9999-99-99",
        must_use_pickle=False,
    ):
        self._move_persons_records_to(
            "notes", ids_from, ids_to, start_date, end_date, must_use_pickle
        )

    def move_persons_files(
        self,
        ids_from,
        ids_to,
        start_date="0000-00-00",
        end_date="9999-99-99",
        must_use_pickle=False,
    ):
        self._move_persons_records_to(
            "files", ids_from, ids_to, start_date, end_date, must_use_pickle
        )

    def _sanitize_ids_from_to(self, ids_from, ids_to):
        id_persons_from = ids_from
        id_persons_to = ids_to

        if isinstance(id_persons_from, list) and isinstance(id_persons_to, list):
            if len(id_persons_from) != len(id_persons_to):
                raise Exception("The lists have different length")
        elif isinstance(id_persons_from, str) and isinstance(id_persons_to, str):
            id_persons_from = [id_persons_from]
            id_persons_to = [id_persons_to]
        else:
            raise Exception("Invalid params type")

        return id_persons_from, id_persons_to

    def _get_unique_ids_from_to(self, ids_from, ids_to):
        return ids_from + list(set(ids_to) - set(ids_from))

    def _move_persons_records_to(
        self,
        r_type,
        ids_from,
        ids_to,
        start_date="0000-00-00",
        end_date="9999-99-99",
        must_use_pickle=False,
    ):
        get_records = {
            "activities": {
                "function": self._get_persons_activities_by_ids,
                "pickle": "persons_activities.p",
            },
            "deals": {
                "function": self._get_persons_deals_by_ids,
                "pickle": "persons_deals.p",
            },
            "notes": {
                "function": self._get_persons_notes_by_ids,
                "pickle": "persons_notes.p",
            },
            "files": {
                "function": self._get_persons_files_by_ids,
                "pickle": "persons_files.p",
            },
        }

        move_recods = {
            "activities": self._move_activity_to,
            "deals": self._move_deal_to,
            "notes": self._move_note_to,
            "files": self._move_file_to,
        }

        id_persons_from, id_persons_to = self._sanitize_ids_from_to(ids_from, ids_to)
        unique_ids = self._get_unique_ids_from_to(ids_from, ids_to)

        if must_use_pickle:
            persons_records = pickle.load(open(get_records[r_type]["pickle"], "rb"))
        else:
            persons_records = get_records[r_type]["function"](
                unique_ids, start_date, end_date
            )
            pickle.dump(persons_records, open(get_records[r_type]["pickle"], "wb"))

        for i in range(0, len(id_persons_from)):
            person_from = id_persons_from[i]
            person_to = id_persons_to[i]
            records = persons_records[person_from]

            for record in records:
                move_recods[r_type](record["id"], person_from, person_to)

            logger.info(
                "Moved {} {} from person id={} to person id={}".format(
                    len(records), r_type, person_from, person_to
                )
            )
            key = input("press enter to keep going: ")
            if key != "":
                return

    def _move_activity_to(self, activity_id, id_person_from, id_person_to):
        res = self._client.activities.update_activity(
            activity_id, {"person_id": id_person_to}
        )

        if res["success"] != True:
            raise Exception(
                "Move activity id={} from person id={} to id={} not succeed!\nResponse:\n{}".format(
                    activity_id, id_person_from, id_person_to, res
                )
            )

    def _move_deal_to(self, deal_id, id_person_from, id_person_to):
        res = self._client.deals.update_deal(str(deal_id), {"person_id": id_person_to})

        if res["success"] == True:
            res = self._client.deals.delete_participant_to_deal_by_person_id(
                deal_id, id_person_from
            )

            if res["success"] != True:
                raise Exception(
                    "Delete person id={} from deal id={} not succeed!\nResponse:\n{}".format(
                        id_person_from, deal_id, res
                    )
                )
        else:
            raise Exception(
                "Move deal id={} from person id={} to id={} not succeed!\nResponse:\n{}".format(
                    deal_id, id_person_from, id_person_to, res
                )
            )

    def _move_note_to(self, note_id, id_person_from, id_person_to):
        res = self._client.notes.update_note(str(note_id), {"person_id": id_person_to})

        if res["success"] != True:
            raise Exception(
                "Move note id={} from person id={} to id={} not succeed!\nResponse:\n{}".format(
                    note_id, id_person_from, id_person_to, res
                )
            )

    def _move_file_to(self, file_id, id_person_from, id_person_to):
        res = self._client.files.update_file(str(file_id), {"person_id": id_person_to})

        if res["success"] != True:
            raise Exception(
                "Move file id={} from person id={} to id={} not succeed!\nResponse:\n{}".format(
                    file_id, id_person_from, id_person_to, res
                )
            )

    def _get_persons_by_ids(self, list_ids, exclude_fields=[]):
        ret = {}
        for person_id in list_ids:
            person_data = self._client.persons.get_person(str(person_id))["data"]

            for field in exclude_fields:
                del person_data[field]

            ret[person_id] = person_data

        return ret

    def _get_persons_deals_by_ids(
        self, list_ids, start_date="0000-00-00", end_date="9999-99-99"
    ):
        filter_data_funct = lambda x: (x["add_time"][:10] >= start_date) and (
            x["add_time"][:10] < end_date
        )
        ret = {}

        for person_id in list_ids:
            all_person_deals = self._client.persons.get_person_deals(
                str(person_id), params={"limit": "500"}
            )["data"]

            if (all_person_deals != None) and (len(all_person_deals) > 0):
                filtered_deals = list(filter(filter_data_funct, all_person_deals))
            else:
                filtered_deals = []

            ret[person_id] = filtered_deals

        return ret

    def _get_persons_activities_by_ids(
        self, list_ids, start_date="0000-00-00", end_date="9999-99-99"
    ):
        filter_data_funct = lambda x: (x["add_time"][:10] >= start_date) and (
            x["add_time"][:10] < end_date
        )
        ret = {}

        for person_id in list_ids:
            all_person_activities = self._client.persons.get_person_activities(
                str(person_id), params={"limit": "500"}
            )["data"]

            if (all_person_activities != None) and (len(all_person_activities) > 0):
                filtered_activities = list(
                    filter(filter_data_funct, all_person_activities)
                )
            else:
                filtered_activities = []

            ret[person_id] = filtered_activities

        return ret

    def _get_persons_notes_by_ids(
        self, list_ids, start_date="0000-00-00", end_date="9999-99-99"
    ):
        filter_data_funct = lambda x: (x["add_time"][:10] >= start_date) and (
            x["add_time"][:10] < end_date
        )

        ret = {id: [] for id in list_ids}

        all_notes = self._client._get_all_from(self._client.notes.get_all_notes)

        if (all_notes != None) and (len(all_notes) > 0):
            filtered_notes = list(filter(filter_data_funct, all_notes))
        else:
            filtered_notes = []

        for note in filtered_notes:
            for person_id in list_ids:
                if str(note["person_id"]) == str(person_id):
                    ret[person_id].append(note)

        return ret

    def _get_persons_files_by_ids(
        self, list_ids, start_date="0000-00-00", end_date="9999-99-99"
    ):
        filter_data_funct = lambda x: (x["add_time"][:10] >= start_date) and (
            x["add_time"][:10] < end_date
        )
        ret = {}

        for person_id in list_ids:
            filter_by_person_funct = lambda x: str(x["person_id"]) == str(person_id)
            all_person_files = self._client.persons.get_person_attaches(
                str(person_id), params={"limit": "500"}
            )["data"]

            if (all_person_files != None) and (len(all_person_files) > 0):
                filtered_files = list(filter(filter_data_funct, all_person_files))

                # Filter by person, because files attached to a deal that is attached to an person is listed too
                filtered_files = list(filter(filter_by_person_funct, filtered_files))
            else:
                filtered_files = []

            ret[person_id] = filtered_files

        return ret



import win32com.client as win32
import pythoncom
from jinja2 import Template

def build_template(template_path, opts={}):
    with open(template_path, "r", encoding='utf8') as f:
        template = Template(f.read())

    return template.render(**opts)


def sender(template_path, sender, to, cc, subject, body_template_opts, send=False):
    """
    Send e-mail using outlook app
    NOTE: if you want to use de default account instead of select, leave 'sender' empty string ""

    Example
    -------
    sender_outlook(
        'my_template.html', #Under template folder
        'my_email@tosend.com.br' # Required
        'to@email1.com;to@email2.com',
        '', #To not use CC leave as empty string
        'Investor reminder – DARF collection',
        {},
        False,
    )
    """

    pythoncom.CoInitialize()

    outlook = win32.Dispatch('outlook.application')
    mail = outlook.CreateItem(0)

    mail.To = to
    mail.CC = cc
    mail.Subject = subject
    mail.HTMLBody = build_template(template_path, body_template_opts)

    for acc in outlook.Session.Accounts:
        if acc.SmtpAddress == sender:
            from_acc = acc

    # mail_item.SendUsingAccount = send_account not working
    # the following statement performs the function instead
    mail._oleobj_.Invoke(*(64209, 0, 8, 0, from_acc))

    if send:
        mail.Send()
    else:
        mail.Display()



from ._investment_funds import InvestmentFunds
from ._onboarding_rede_customers import OnboardingRedeCustomers
from ._pension_funds import PensionFunds
from ._rede_advisors import RedeAdvisors
from ._rede_customer import RedeCustomer

import requests
import json

class Client():
    """
    Core methods to access xp api
    """
    def __init__(self, access_token):
        self.access_token = access_token

        self.investment_funds = InvestmentFunds(self)
        self.onboarding_rede_customers = OnboardingRedeCustomers(self)
        self.pension_funds = PensionFunds(self)
        self.rede_advisors = RedeAdvisors(self)
        self.rede_customer = RedeCustomer(self)


    def request(self, url, params={}, subscription_key="", token_type="Bearer"):
        r = requests.get(
            url="https://" + url,
            params=params,
            headers={
                "Authorization": "{} {}".format(token_type, self.access_token),
                "ocp-apim-subscription-key": subscription_key,
                "Host": "api.xpi.com.br",
                "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.114 Safari/537.36",
                "Accept": "application/json",
                "accept-language": "en-US,en-CA;q=0.9,en;q=0.8,hi-IN;q=0.7,hi;q=0.6",
            }
        )

        try:
            return json.loads(r.text)
        except Exception as e:
            print("Error at request to xp api")
            raise e



import re
import pandas as pd
import numpy as np


class Sanitizer:
    def __init__(self, client):
        self._client = client

    def get_duplicates(self):
        all_persons = self._get_all_persons()

        all_persons_filtered = []
        unique_fields = ["name", "xp_code", "email"]
        duplicated_persons = {}

        for person in all_persons:
            all_persons_filtered.append(
                {
                    "id": person["id"],
                    "name": person["name"].lower(),
                    "xp_code": person["9d6794606b66471270c5ae7437fa8a58b3a6f224"],
                    "email": person["email"][0]["value"].lower(),
                }
            )

        df = pd.DataFrame(all_persons_filtered).set_index("id")

        df = df.replace("", np.nan)
        df["xp_code"] = df["xp_code"].replace(0, np.nan)
        df["email"] = df["email"].replace("-", np.nan)

        for field in unique_fields:
            df_not_nan = df.loc[df[field].notna()]
            duplicated_in_this_field = df_not_nan.loc[
                df_not_nan[field].duplicated(keep=False)
            ]
            duplicated_persons[field] = duplicated_in_this_field.sort_values([field])

        return duplicated_persons

    def get_invalid_names(self):
        all_persons = self._get_all_persons()

        invalid_persons = []

        for person in all_persons:
            name = person["name"]
            string_check = re.compile(r"[@_!#$%^&*()<>?/\|}{~:]")

            if string_check.search(name) != None:
                invalid_persons.append(person)

        return invalid_persons

    def _get_all_persons(self):
        return self._client._get_all_from(self._client.persons.get_all_persons)



from setuptools import setup, find_packages

with open("README.md", "r") as file:
    long_description = file.read()

setup(
    name='gsozo-pkg',
    version='0.0.1',
    author="Tecnologia Grupo SOZO",
    author_email="tecnologia@gruposozo.com.br",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
        classifiers=[
        "Programming Language :: Python :: 3",
    ],
    install_requires=[
        'pipedrive-python-lib',
        'loguru',
        'numpy',
        'pandas',
        'sqlalchemy',
        'tqdm',
        'jinja2',
        'pywin32 ; platform_system=="Windows"'
    ],
    include_package_data=True,
    python_requires='>=3.8',
)



class OnboardingRedeCustomers():
    """
    Access 'onboarding rede customers' api.
    Have some methods to direct access some endpoints.
    """

    def __init__(self, client):
        self.client = client
        self.base_url = 'api.xpi.com.br/onboarding-rede-customers'
        self.subscription_key = 'aaa32b63cff04fc2aa6c4324fcd06583'


    def get_from_url(self, api_path, params={}):
        '''
        Get something from the url 'api.xpi.com.br/onboarding-rede-customers'.
        You don't need to pass the base route, only the path for this api.
        The path must start with '/'.

        e.g.: get_from_url('/api/v1/rede/customer/details/code/{customer_code}', params)
        '''

        full_url = self.base_url + api_path
        return self.client.request(full_url, params, self.subscription_key)


    def get_customer_details(self, xp_code, params={}):
        '''
        Get customer wallet from xp api

        e.g.: get_customer_details(
            123321,             # Customer xp code. Can be string or int
            params              # A dict with other params (not full mapped yet)
        )
        '''
        return self.get_from_url(
            '/api/v1/rede/customer/details/code/{}'.format(str(xp_code)),
            params
        )



from pipedrive.organizations import Organizations as BasePersons


class Organizations(BasePersons):
    def __init__(self, client):
        super().__init__(client)

    def add_follower_to_organization(self, org_id, user_id, **kwargs):
        url = "organizations/{}/followers".format(org_id)
        data = { "user_id": user_id }
        return self._client._post(self._client.BASE_URL + url, json=data, **kwargs)



from pipedrive.persons import Persons as BasePersons


class Persons(BasePersons):
    def __init__(self, client):
        super().__init__(client)

    def get_person_activities(self, person_id, **kwargs):
        url = "persons/{}/activities".format(person_id)
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def get_person_attaches(self, person_id, **kwargs):
        url = "persons/{}/files".format(person_id)
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def get_person_emails(self, person_id, **kwargs):
        url = "persons/{}/mailMessages".format(person_id)
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def get_all_persons(self, *args, pagination_on=True, **kwargs):
        if pagination_on:
            return super().get_all_persons(*args, **kwargs)
        else:
            return self._client._get_all_from(super().get_all_persons, *args, **kwargs)

    def get_person_one_field(self, field_id, **kwargs):
        url = "personFields/{}".format(field_id)
        return self._client._get(self._client.BASE_URL + url, **kwargs)

    def get_person_followers(self, person_id, pagination_on=True, **kwargs):
        url = "persons/{}/followers".format(person_id)
        get_function = lambda **kwargs: self._client._get(self._client.BASE_URL + url, **kwargs)

        if pagination_on:
            return get_function()
        else:
            return self._client._get_all_from(get_function, **kwargs)

    def add_follower_to_person(self, person_id, user_id, **kwargs):
        url = "persons/{}/followers".format(person_id)
        data = { "user_id": user_id }
        return self._client._post(self._client.BASE_URL + url, json=data, **kwargs)

    def update_person_field(self, field_id, data, **kwargs):
        url = "personFields/{}".format(field_id)
        return self._client._put(self._client.BASE_URL + url, json=data, **kwargs)

    def delete_follower_to_person(self, person_id, user_id, **kwargs):
        followers = self.get_person_followers(person_id, pagination_on=False)
        follower_id = next((x['id'] for x in followers if x['user_id'] == user_id), None)

        if follower_id:
            url = "persons/{}/followers/{}".format(person_id, follower_id)
            return self._client._delete(self._client.BASE_URL + url, **kwargs)

    def merge_two_persons(self, person_to_keep, person_to_overwrite, **kwargs):
        url = "persons/{}/merge".format(person_to_overwrite)
        data = { "merge_with_id": person_to_keep }
        return self._client._put(self._client.BASE_URL + url, json=data, **kwargs)



from pipedrive.client import Client as BaseClient

from ._sanitizer import Sanitizer
from ._manipulator import Manipulator
from ._backup import Backup

from .persons import Persons
from .deals import Deals
from .files import Files
from .activities import Activities
from .organizations import Organizations
from .stages import Stages

import copy

#from loguru import logger


class Client(BaseClient):
    def __init__(self, company_domain, api_token):
        super().__init__(domain="https://{}.pipedrive.com/".format(company_domain))
        super().set_api_token(api_token)

        self.persons = Persons(self)
        self.deals = Deals(self)
        self.files = Files(self)
        self.activities = Activities(self)
        self.organizations = Organizations(self)
        self.stages = Stages(self)

        self.sanitizer = Sanitizer(self)
        self.manipulator = Manipulator(self)
        self.backup = Backup(self)

    """
        Loop through all pages on record (because the pipe limit for a page is 500), getting all the records
        NOTE: different of the get_all from each record like persons.get_all_persons(), this function return direct the data

        Parameters
        ----------
        funct: function
            function of the record that will be looped through

        extra_params: dict
            extra params to be passed to the function besides the 'limit' and 'start', used for the pagination
    """

    def _get_all_from(self, func, *args, **kwargs):
        data = []
        start = 0
        _args = copy.deepcopy(list(args))
        _kwargs = copy.deepcopy(kwargs)

        if len(_args) > 0 and type(_args[0]) == dict:
            params = _args[0]
        elif len(_args) > 1 and type(_args[1]) == dict:
            params = _args[1]
        elif _kwargs.get('params'):
            params = _kwargs['params']
        else:
            _kwargs['params'] = {}
            params = _kwargs['params']

        params.update({"limit": "500", "start": start})

        while True:
            res = func(*_args, **_kwargs)

            data = data + res["data"] if res['data'] else data
            if not res["additional_data"]["pagination"]["more_items_in_collection"]:
                break
            else:
                params['start'] = res["additional_data"]["pagination"]["next_start"]

        return data
